'use strict'

const fetch = require('node-fetch');
const moment = require("moment");

module.exports = class SignalServer {

    signalserver_host;
    signalserver_key;
    #keys = {
        connection: 'connection',
        token: 'token'
    }

    constructor(signalserver_host, signalserver_key) {
        try {
            this.SetHost(signalserver_host);
            this.SetApiKey(signalserver_key);
        } catch (e) {
            throw e;
        }
    }

    SetHost(signalserver_host) {
        try {
            if(!signalserver_host || typeof signalserver_host !== 'string') throw `Invalid signalserver_host: ${signalserver_host}`;
            this.signalserver_host = signalserver_host;
        } catch (e) {
            throw e;
        }
    }

    SetApiKey(signalserver_key) {
        try {
            this.signalserver_key = signalserver_key;
        } catch (e) {
            throw e;
        }
    }

    async Fetch(method = 'GET', type, data = {}) {
        try {
            let body, headers;
            let urlParamList = [];
            let urlParams = '';
            if(this.signalserver_key) data.API_KEY = this.signalserver_key;
            const keys = Object.keys(data);

            switch (method) {
                case 'GET':
                    keys.forEach(key => { urlParamList.push(`${key}=${data[key]}`) });
                    if(urlParamList.length > 0) urlParams = `?${urlParamList.join('&')}`;
                    break;
                case 'POST':
                        body = JSON.stringify(data);
                        headers = {'Content-Type': 'application/json;charset=utf-8'};
                    break;
                default: throw {
                    msg: 'Invalid method.',
                    code: 'ERR_FETCHMETHOD'
                };
            }

            return await fetch(`${this.signalserver_host}/${type}${urlParams}`, {
                method: method,
                headers: headers,
                body: body,
            })
                .then(res => {
                   if(!res.ok) throw res;
                    return res.json()
                })
                .then(res => {
                    if(!res) throw res;
                    return res;
                })
                .catch(e => {
                    throw e;
                });
        } catch (e) {
            throw e;
        }
    }

    async #Get(type, data) {
        try {
            return await this.Fetch('GET', type, data);
        } catch (e) {
            throw e;
        }
    }

    async #Set(type, key, value) {
        try {
            const data = {
                key,
                value
            };

            return await this.Fetch('POST', type, data);
        } catch (e) {
            throw e;
        }
    }

    /**
     * 커넥션 등록 (offer)
     * @param connectionId
     * @param offer
     * @param answer
     * @returns {Promise<*|undefined>}
     * @constructor
     */
    async SetConnection({connectionId, offer = null, answer = null}) {
        try {
            const data = {
                offer,
                answer,
                timestamp: moment.now()
            };

            return await this.#Set(this.#keys.connection, connectionId, data).then(res => {
                return connectionId;
            }).catch(e => {
                throw e;
            });
        } catch (e) {
            throw e;
        }
    }

    /**
     * 커넥션 정보 조회 (answer)
     * @returns {Promise<*|undefined>}
     * @constructor
     * @param connectionId
     */
    async GetConnection(connectionId) {
        try {
            return (await this.#Get(this.#keys.connection, { connectionId })).data.result;
        } catch (e) {
            throw e;
        }
    }

    /**
     * 토큰 유효성 조회
     * @param token
     * @param address
     * @returns {Promise<*|undefined>}
     * @constructor
     */
    async GetTokenValidity(token, address) {
        try {
            return await this.#Get(this.#keys.token, { token, address });
        } catch (e) {
            throw e;
        }
    }

}