'use strict'

module.exports = class Chunk {

    static maxDataSize = 200 * 1024;
    order;
    chunk;
    chunkOrder;
    chunkLength;

    /**
     *
     * @param chunk: chunked fragment of full data sequence
     * @param order: sequence order
     * @param chunkOrder: chunk group order
     * @param chunkLength: chunk group length
     */
    constructor({chunk, order, chunkOrder, chunkLength}) {
        try {
           if(!Chunk.IsChunk({chunk, order, chunkOrder, chunkLength})) throw `Invalid chunk data`;
           this.chunk = chunk;
           this.order = order;
           this.chunkOrder = chunkOrder;
           this.chunkLength = chunkLength;
        } catch (e) {
            throw e;
        }
    }

    static IsChunk(obj) {
        try {
            if(typeof obj !== 'object') throw `Invalid chunk object type: ${typeof obj.chunk}`;
            if(!obj.chunk || typeof obj.chunk !== 'string') throw `Invalid chunk type: ${typeof obj.chunk}`;
            if(!obj.order || typeof obj.order !== 'number') throw `Invalid order type: ${typeof obj.order}`;
            if(!obj.chunkOrder || typeof obj.chunkOrder !== 'number') throw `Invalid chunkOrder type: ${typeof obj.chunkOrder}`;
            if(!obj.chunkLength || typeof obj.chunkLength !== 'number') throw `Invalid chunkLength type: ${typeof obj.chunkLength}`;
            if(obj.chunkOrder + 1 > obj.chunkLength) throw `Invalid chunk order`;
            if(JSON.stringify({
                chunk: obj.chunk,
                order: obj.order,
                chunkOrder: obj.chunkOrder,
                chunkLength: obj.chunkLength,
            }).length > this.maxDataSize) throw 'Max size exceeded.';

            return true;
        } catch (e) {
            return false;
        }
    }

}