'use strict'

const Encoder = require("../util/encoder");
const Encryptor = require("../util/encryptor");

module.exports = class ConnectionData {

    method;
    data;
    encoded;
    encrypted;
    static #allowed = ['ping', 'connect', 'error', 'chunk', 'offer', 'answer', 'request', 'response', 'disconnect'];
    static #maxDataSize = 220 * 1024;

    constructor({method, data, encode = false, encrypt = false, password = null}) {
        try {
            if(typeof method !== 'string') throw `Invalid method type: ${typeof method}`;
            if(typeof data !== 'object') throw `Invalid data type: ${typeof data}`;
            if(!data.hasOwnProperty('type')) throw `data must be a child of ConnectionDataBodyFactory`;
            if(typeof data.Object !== 'function' || typeof data.Stringify !== 'function') throw `data must be a child of ConnectionDataBodyFactory`;
            if(!ConnectionData.#allowed.includes(method)) throw `Invalid method: ${method}`;
            if(!password && encrypt === true) throw `Cannot encrypt without password`;

            data = data.Object();
            if(encode === true) data = Encoder.Stringify(data);
            if(password && encrypt === true) {
                if(encrypt === true) data = Encryptor.Encrypt(data, password);
                else {
                    encode = true;
                    data = Encryptor.Encrypt(Encoder.Stringify(data), password);
                }
            }
            if(JSON.stringify({ method, data, encode, encrypt, password }).length > ConnectionData.#maxDataSize) throw 'Max size exceeded.';

            this.method = method;
            this.data = data;
            this.encoded = encode;
            this.encrypted = encrypt;
        } catch (e) {
            throw {
                msg: e,
                code: 'ERR_CONNECTIONDATA'
            };
        }
    }

    Object() {
        return {
            method: this.method,
            data: this.data,
            encoded: this.encoded,
            encrypted: this.encrypted,
        };
    }

    Stringify() {
        try {
            return JSON.stringify(this.Object());
        } catch (e) {
            throw e;
        }
    }

}